using System;

using Microsoft.Xna.Framework.Input;

using Syntriax.Engine.Core;
using Syntriax.Engine.Systems.Input;

namespace Pong.Behaviours;

public class PongManagerBehaviour : Behaviour
{
    public Action<PongManagerBehaviour>? OnReset { get; set; } = null;
    public Action<PongManagerBehaviour>? OnFinished { get; set; } = null;
    public Action<PongManagerBehaviour>? OnScored { get; set; } = null;

    public int ScoreLeft { get; private set; } = 0;
    public int ScoreRight { get; private set; } = 0;
    public int ScoreSum => ScoreLeft + ScoreRight;

    public int WinScore { get; } = 5;

    public PongManagerBehaviour() => WinScore = 5;
    public PongManagerBehaviour(int winScore) => WinScore = winScore;

    protected override void OnFirstActiveFrame()
    {
        var buttonInputs = Universe.FindRequiredBehaviour<IButtonInputs<Keys>>();
        buttonInputs.RegisterOnRelease(Keys.Space, (_, _1) => Reset());
    }


    public void ScoreToLeft()
    {
        ScoreLeft++;
        OnScored?.InvokeSafe(this);

        CheckFinish();
    }

    public void ScoreToRight()
    {
        ScoreRight++;
        OnScored?.InvokeSafe(this);

        CheckFinish();
    }

    public void Reset()
    {
        ScoreLeft = ScoreRight = 0;
        OnReset?.InvokeSafe(this);
    }

    private void CheckFinish()
    {
        int halfwayScore = (int)(WinScore * .5f);

        if (ScoreLeft > halfwayScore || ScoreRight > halfwayScore)
            OnFinished?.InvokeSafe(this);
    }
}
